
/* IMPORT */

import Oniguruma from './textmate/oniguruma';
import Registry from './textmate/registry';
import Renderer from './textmate/renderer';
import Tokenizer from './textmate/tokenizer';
import type {Disposer, PromiseMaybe, AbortSignal, OptionsHighlight, OptionsHighMate, TextMateToken} from './types';

/* MAIN */

class HighMate<Grammar extends string = string, Theme extends string = string> {

  /* VARIABLES */

  private oniguruma: Oniguruma;
  private registry: Registry<Grammar, Theme>;
  private tokenizer: Tokenizer<Grammar, Theme>;

  /* CONSTRUCTOR */

  constructor ( options: OptionsHighMate<Grammar, Theme> ) {

    const {getGrammar, getOniguruma, getTheme} = options;

    this.oniguruma = new Oniguruma ({ getOniguruma });
    this.registry = new Registry ({ getGrammar, getTheme, getOniguruma: () => this.oniguruma.load () });
    this.tokenizer = new Tokenizer ( this.registry );

  }

  /* API */

  highlightToAbstract = async <T> ( options: OptionsHighlight<Grammar, Theme>, highlighter: ( lines: string[], tokens: TextMateToken[][], background: string ) => PromiseMaybe<T> ): Promise<T> => {

    const lines = options.code.replace ( /^(\r?\n|\r)+|(\r?\n|\r)+$/g, '' ).split ( /\r?\n|\r/g );
    const tokens: TextMateToken[][] = [];

    await this.tokenize ( lines, options.grammar, options.theme, options.signal, ( lineTokens, lineIndex ) => {
      tokens[lineIndex] = lineTokens;
    });

    const background = options.background || ( await this.registry.loadThemeColorsDefault ( options.theme ) ).background;
    const result = highlighter ( lines, tokens, background );

    return result;

  };

  highlightToDOM = ( options: OptionsHighlight<Grammar, Theme> ): Promise<HTMLPreElement> => {

    return this.highlightToAbstract ( options, Renderer.toDOM );

  };

  highlightToHighlights = ( options: OptionsHighlight<Grammar, Theme> ): Promise<[HTMLPreElement, Disposer]> => {

    return this.highlightToAbstract ( options, Renderer.toHighlights );

  };

  highlightToHTML = ( options: OptionsHighlight<Grammar, Theme> ): Promise<string> => {

    return this.highlightToAbstract ( options, Renderer.toHTML );

  };

  tokenize = ( lines: string[], grammar: Grammar, theme: Theme, signal: AbortSignal | undefined, onTokens: ( tokens: TextMateToken[], lineIndex: number ) => void ): Promise<void> => {

    return this.tokenizer.tokenize ( lines, grammar, theme, signal, onTokens );

  };

}

/* EXPORT */

export default HighMate;
